// Yasuo Hirose and Takushi Kurozumi
// "Changes in the Federal Reserve Communication Strategy: A Structural Investigation"
// Replication code for robustness exercise (Section 4.3)

var
 l_obs r_obs pi_obs dy_obs dc_obs di_obs dw_obs
 mup muw zcap rk k pk c inve y lab pinf w r a  b g qs
 ms spinf sw kp yf
 n1 n2 n3 n4 n5 n6 n7
 r4 r8 r4_obs r8_obs
 tvrp4 tvrp8
 ;

varexo
 ea eb eg eqs em epinf ew
 en1 en2 en3 en4 en5 en6 en7
 eta4 eta8
 ;

parameters
 ctou clandaw cg curvp curvw
 constelab constebeta calfa
 czcap csadjcost csigma chabb ccs cinvs cfc 
 cindw cprobw cindp cprobp csigl crpi cry crr 
 crhoa crhoas crhob crhog crhols crhoqs crhoms crhopinf crhow  
 ctrend constepinf
 c4 c8
 crho4 crho8
 ;

// fixed parameters
ctou=.025;
clandaw=1.5;
cg=0.18;
curvp=10;
curvw=10;

crhoms=0;

model(linear); 

// derived from steady state
#cpie=1+constepinf/100;
#cgamma=1+ctrend/100 ;
#cbeta=1/(1+constebeta/100);
#clandap=cfc;
#cbetabar=cbeta*cgamma^(-csigma);
#cr=cpie/(cbeta*cgamma^(-csigma));
#crk=(cbeta^(-1))*(cgamma^csigma) - (1-ctou);
#cw = (calfa^calfa*(1-calfa)^(1-calfa)/(clandap*crk^calfa))^(1/(1-calfa));
#cikbar=(1-(1-ctou)/cgamma);
#cik=(1-(1-ctou)/cgamma)*cgamma;
#clk=((1-calfa)/calfa)*(crk/cw);
#cky=cfc*(clk)^(calfa-1);
#ciy=cik*cky;
#ccy=1-cg-cik*cky;
#crkky=crk*cky;
#cwhlc=(1/clandaw)*(1-calfa)/calfa*crk*cky/ccy;
#cwly=1-crk*cky;
#conster=(cr-1)*100;

y = ccy*c + ciy*inve + crkky*zcap + g;

c = (chabb/cgamma)/(1+chabb/cgamma)*(c(-1)-a) + 1/(1+chabb/cgamma)*(c(+1)+a(+1))
   + (csigma-1)*cwhlc/(csigma*(1+chabb/cgamma))*(lab-lab(+1))
   - (1-chabb/cgamma)/(csigma*(1+chabb/cgamma))*(r-pinf(+1)+b);

inve = 1/(1+cbetabar*cgamma)*(inve(-1)-a) + cbetabar*cgamma/(1+cbetabar*cgamma)*(inve(1)+a(+1))
      + (1/(cgamma^2*csadjcost*(1+cbetabar*cgamma)))*pk + qs;

pk = (1-ctou)/(crk+1-ctou)*pk(+1) + crk/(crk+1-ctou)*rk(+1) - (r-pinf(+1)+b);

y = cfc*(calfa*(k-a)+(1-calfa)*lab);

k = kp(-1) + zcap;

zcap =  (1-czcap)/czcap*rk;

kp = (1-cikbar)*(kp(-1)-a) + cikbar*(inve + cgamma^2*csadjcost*(1+cbetabar*cgamma)*qs);

mup = calfa*(k-lab-a)-w;

pinf = cindp/(1+cbetabar*cgamma*cindp)*pinf(-1) + cbetabar*cgamma/(1+cbetabar*cgamma*cindp)*pinf(+1) 
      -(1-cprobp)*(1-cbetabar*cgamma*cprobp)/(cprobp*(1+cbetabar*cgamma*cindp)*((cfc-1)*curvp+1))*mup
      + spinf;

rk = - (k - lab - a) + w;

muw = w - (csigl*lab+1/(1-chabb/cgamma)*(c-chabb/cgamma*(c(-1)-a)));

w = 1/(1+cbetabar*cgamma)*(w(-1)-a) + cbetabar*cgamma/(1+cbetabar*cgamma)*(w(+1)+a(+1)+pinf(+1))
   - (1+cbetabar*cgamma*cindw)/(1+cbetabar*cgamma)*pinf + cindw/(1+cbetabar*cgamma)*pinf(-1)
   -(1-cprobw)*(1-cbetabar*cgamma*cprobw)/(cprobw*(1+cbetabar*cgamma)*((clandaw-1)*curvw+1))*muw + sw;

r = crr*r(-1) + (1-crr)*(crpi*(pinf+pinf(-1)+pinf(-2)+pinf(-3))/4+cry*(y-yf)) + ms;

yf = -cfc*calfa*a;

a = crhoa*a(-1)  + ea;
b = crhob*b(-1) + eb;
g = crhog*g(-1) + eg;
qs = crhoqs*qs(-1) + eqs;
spinf = crhopinf*spinf(-1) + epinf;
sw = crhow*sw(-1) + ew; 

// anticipated and unanticipated monetary policy disturbances
ms = crhoms*ms(-1) + em
     + n1(-1) + n2(-2) + n3(-3) + n4(-4) + n5(-5) + n6(-6) + n7(-7);

n1 = en1;
n2 = en2;
n3 = en3;
n4 = en4;
n5 = en5;
n6 = en6;
n7 = en7;

r4 = (r+r(+1)+r(+2)+r(+3))/4;
r8= (r+r(+1)+r(+2)+r(+3)+r(+4)+r(+5)+r(+6)+r(+7))/8;

// time-varying components of term premiums
tvrp4 = crho4*tvrp4(-1) + eta4;
tvrp8 = crho8*tvrp8(-1) + eta8;

// measurment equations

dy_obs = ctrend + a + y - y(-1);
dc_obs = ctrend + a + c - c(-1);
di_obs = ctrend + a + inve - inve(-1);
dw_obs = ctrend + a + w - w(-1);
l_obs = constelab + lab;
pi_obs = constepinf + pinf;
r_obs = conster + r;
r4_obs = conster + c4 + r4 + tvrp4;
r8_obs = conster + c8 + r8 + tvrp8;

end; 


estimated_params;

crhoa,         0.06,,,  BETA_PDF,      0.5,   0.2;
crhob,         0.97,,,  BETA_PDF,      0.5,   0.2;
crhog,         0.98,,,  BETA_PDF,      0.5,   0.2;
crhoqs,        0.72,,,  BETA_PDF,      0.5,   0.2;
crhopinf,      0.41,,,  BETA_PDF,      0.5,   0.2;
crhow,         0.23,,,  BETA_PDF,      0.5,   0.2;
csadjcost,     7.08,,,  NORMAL_PDF,    4,     1.5;
csigma,        1.43,,,  NORMAL_PDF,    1.5,   0.375;
chabb,         0.60,,,  BETA_PDF,      0.7,   0.1;
cprobw,        0.89,,,  BETA_PDF,      0.5,   0.1;
csigl,         1.47,,,  NORMAL_PDF,    2,     0.75;
cprobp,        0.87,,,  BETA_PDF,      0.5,   0.10;
cindw,         0.39,,,  BETA_PDF,      0.5,   0.15;
cindp,         0.21,,,  BETA_PDF,      0.5,   0.15;
czcap,         0.75,,,  BETA_PDF,      0.5,   0.15;
cfc,           1.40,,,  NORMAL_PDF,    1.25,  0.125;
crpi,          1.64,,,  NORMAL_PDF,    1.5,   0.25;
crr,           0.94,,,  BETA_PDF,      0.75,  0.10;
cry,           0.14,,,  NORMAL_PDF,    0.125, 0.05;
constepinf,    0.65,,,  GAMMA_PDF,     0.625, 0.1;
constebeta,    0.18,,,  GAMMA_PDF,     0.25,  0.1;
constelab,     0.50,,,  NORMAL_PDF,    0,     2;
ctrend,        0.41,,,  NORMAL_PDF,    0.4,   0.10;
calfa,         0.18,,,  NORMAL_PDF,    0.3,   0.05;
c4,            0.04,,,  NORMAL_PDF,    0.03,  0.05;
c8,            0.11,,,  NORMAL_PDF,    0.10,  0.05;
crho4, ,,,  BETA_PDF,      0.5,   0.2;
crho8, ,,,  BETA_PDF,      0.5,   0.2;

stderr ea,     0.73,,,  INV_GAMMA_PDF, 0.1,   2;
stderr eb,     0.15,,,  INV_GAMMA_PDF, 0.1,   2;
stderr eg,     0.38,,,  INV_GAMMA_PDF, 0.1,   2;
stderr eqs,    0.34,,,  INV_GAMMA_PDF, 0.1,   2;
stderr em,     0.10,,,  INV_GAMMA_PDF, 0.1,   2;
stderr epinf,  0.11,,,  INV_GAMMA_PDF, 0.1,   2;
stderr ew,     0.27,,,  INV_GAMMA_PDF, 0.1,   2;

//sqrt(0.1*0.1/7)=0.0378
stderr en1,    0.04,,,   INV_GAMMA_PDF, 0.038, 2;
stderr en2,    0.08,,,   INV_GAMMA_PDF, 0.038, 2;
stderr en3,    0.07,,,   INV_GAMMA_PDF, 0.038, 2;
stderr en4,    0.02,,,   INV_GAMMA_PDF, 0.038, 2;
stderr en5,    0.02,,,   INV_GAMMA_PDF, 0.038, 2;
stderr en6,    0.02,,,   INV_GAMMA_PDF, 0.038, 2;
stderr en7,    0.05,,,   INV_GAMMA_PDF, 0.038, 2;

stderr eta4,     ,,,  INV_GAMMA_PDF, 0.1,   2;
stderr eta8,     ,,,  INV_GAMMA_PDF, 0.1,   2;

end;

varobs dy_obs dc_obs di_obs dw_obs l_obs pi_obs r_obs r4_obs r8_obs;

estimation(
datafile = usdata,
mode_check,
mh_replic = 500000,
mh_nblocks = 3,
mh_jscale = 0.34)
;

stoch_simul(irf=0);
shock_decomposition(parameter_set=posterior_mean) ms;

save('results_all.mat');
 